@@ -117,6 +117,9 @@ class mapTiles(ranaModule):
     self.downloadRequestTimeout = 30 # in seconds
     self.startTileDownloadManagementThread()
     self.startTileLoadingThread()
+
+    self.shutdownAllThreads = False # notify the threads that shutdown is in progress
+    
     specialTiles = [
                     ('tileDownloading' , 'icons/bitmap/tile_downloading.png'),
                     ('tileDownloadFailed' , 'icons/bitmap/tile_download_failed.png'),
@@ -134,8 +137,8 @@ class mapTiles(ranaModule):
 
   def startTileDownloadManagementThread(self):
     """start the consumer thread for download requests"""
-    t = Thread(target=self.tileDownloadManager)
-    t.daemon = True # we need that the worker dies with the program
+    t = Thread(target=self.tileDownloadManager, name='automatic tile download management thread')
+    t.setDaemon(True) # we need that the worker dies with the program
     t.start()
 
   def tileDownloadManager(self):
@@ -147,6 +150,9 @@ class mapTiles(ranaModule):
           wait for notification that there might be download requests or free download slots
           """
           self.threadlListCondition.wait()
+          if self.shutdownAllThreads:
+            print "\nmapTiles: automatic tile download management thread shutting down"
+            break
           with self.downloadRequestPoolLock:
             activeThreads = len(self.threads)
             maxThreads = int(self.get("maxAutoDownloadThreads", 20))
@@ -160,6 +166,7 @@ class mapTiles(ranaModule):
                   (name,x,y,z,layer,layerPrefix,layerType, filename, folder, timestamp) = request
                   # start a new thread
                   self.threads[name] = self.tileDownloader(name,x,y,z,layer,layerPrefix,layerType, filename, folder, self)
+                  self.threads[name].daemon = True
                   self.threads[name].start()
                   # chenge the status tile to "Downloading..."
                   with self.imagesLock:
@@ -190,8 +197,8 @@ class mapTiles(ranaModule):
 
   def startTileLoadingThread(self):
     """start the loading-request consumer thread"""
-    t = Thread(target=self.tileLoader)
-    t.daemon = True # we need that the worker dies with the program
+    t = Thread(target=self.tileLoader, name='tile loading thread')
+    t.setDaemon(True) # we need that the worker dies with the program
     t.start()
 
   def tileLoader(self):
@@ -202,10 +209,9 @@ class mapTiles(ranaModule):
       if key == 'loadRequest':
         (name, x, y, z, layer) = args
         self.loadImage(name, x, y, z, layer)
-
-  def firstTime(self):
-    # the config folder should set the tile folder path by now
-    self.tileFolder = self.get('tileFolder', 'cache/images')
+      elif key == 'shutdown':
+        print "\nmapTiles: tile loading thread shutting down"
+        break
 
   def loadSpecialTiles(self, specialTiles):
     """load special tiles from files to the special tiles cache"""
@@ -470,37 +476,46 @@ class mapTiles(ranaModule):
     if(layerInfo == None): # is the layer info valid
       return('NOK')
 
-    layerType = layerInfo.get('type','png')
     layerPrefix = layerInfo.get('folderPrefix','OSM')
+    layerType = layerInfo.get('type','png')
 
-    filename = self.tileFolder + (self.imagePath(x,y,z,layerPrefix, layerType))
-
-    storageType = self.get('tileStorageType', 'files')
-    if storageType == 'sqlite': # use the sqlite based storage method
-      m = self.m.get('storeTiles', None) # get the tile storage module
-      if m:
-        try:
-          buffer = m.getTile(layerPrefix, z, x, y, layerType)
-          if buffer:
-            pl = gtk.gdk.PixbufLoader()
-            pl.write(buffer)
-            pl.close()
-            pixbuf = pl.get_pixbuf()
-            self.storeInMemmory(self.pixbufToCairoImageSurface(pixbuf), name)
-            return('OK')
-        except Exception, e:
-          print "loading tile from sqlite failed"
-          print "exception: ", e
-    else: #use the default method -> load from files
-      if (os.path.exists(filename)):
-        try:
-          pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
-          self.storeInMemmory(self.pixbufToCairoImageSurface(pixbuf), name)
-        except Exception, e:
-          print "the tile image is corrupted nad/or there are no tiles for this zoomlevel, exception:\n%s" % e
+    storeTiles = self.m.get('storeTiles', None) # get the tile storage module
+    if storeTiles:
+      pixbuf = storeTiles.getTile(layerPrefix, z, x, y, layerType)
+      """None from getTiles means the tile was not found
+         False means loading the tile from file to pixbuf failed"""
+      if pixbuf:
+        self.storeInMemmory(self.pixbufToCairoImageSurface(pixbuf), name)
         return('OK')
 
-    # Image not found anywhere - resort to downloading it
+    filename = self.getTileFolderPath() + (self.getImagePath(x,y,z,layerPrefix, layerType))
+#
+#    storageType = self.get('tileStorageType', 'files')
+#    if storageType == 'sqlite': # use the sqlite based storage method
+#      m = self.m.get('storeTiles', None) # get the tile storage module
+#      if m:
+#        try:
+#          buffer = m.getTile(layerPrefix, z, x, y, layerType)
+#          if buffer:
+#            pl = gtk.gdk.PixbufLoader()
+#            pl.write(buffer)
+#            pl.close()
+#            pixbuf = pl.get_pixbuf()
+#            self.storeInMemmory(self.pixbufToCairoImageSurface(pixbuf), name)
+#            return('OK')
+#        except Exception, e:
+#          print "loading tile from sqlite failed"
+#          print "exception: ", e
+#    else: #use the default method -> load from files
+#      if (os.path.exists(filename)):
+#        try:
+#          pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
+#          self.storeInMemmory(self.pixbufToCairoImageSurface(pixbuf), name)
+#        except Exception, e:
+#          print "the tile image is corrupted nad/or there are no tiles for this zoomlevel, exception:\n%s" % e
+#        return('OK')
+
+    # Image not found anywhere locally - resort to downloading it
 
     # Are we allowed to download it ? (network=='full')
     if(self.get('network','full')=='full'):
@@ -508,7 +523,7 @@ class mapTiles(ranaModule):
       """the thread list condition is used to signalize to the download manager,
       that here is a new download request"""
       with self.threadlListCondition:
-        folder = self.tileFolder + self.imageFolder(x, z, layerPrefix) # target folder
+        folder = self.getTileFolderPath() + self.getImageFolder(x, z, layerPrefix) # target folder
         timestamp = time.time()
         request = (name,x,y,z,layer,layerPrefix,layerType, filename, folder, timestamp)
 
@@ -541,6 +556,16 @@ class mapTiles(ranaModule):
     ''' surface now contains the image in a Cairo surface '''
     self.storeInMemmory(surface, name, type, expireTimestamp, dictIndex)
 
+  def filePath2Pixbuf(self, filePath):
+    """return a pixbaf for a given filePath"""
+    try:
+      pixbuf = gtk.gdk.pixbuf_new_from_file(filePath)
+      return(pixbuf)
+    except Exception, e:
+      print "the tile image is corrupted nad/or there are no tiles for this zoomlevel, exception:\n%s" % e
+      return False
+
+
   def storeInMemmory(self, surface, name, type="normal", expireTimestamp=None, dictIndex=0):
     """store a given image surface in the memmory image cache
        dictIndex = 0 -> nromal map tiles + tile specific error tiles
@@ -610,16 +635,20 @@ class mapTiles(ranaModule):
     (suitable for use as part of filenames, dictionary keys, etc)"""
     return("%s_%d_%d_%d" % (layer,z,x,y))
 
-  def imagePath(self,x,y,z,prefix, extension):
+  def getImagePath(self,x,y,z,prefix, extension):
     """Get a unique name for a tile image
     (suitable for use as part of filenames, dictionary keys, etc)"""
     return("%s/%d/%d/%d.%s" % (prefix,z,x,y,extension))
-
-  def imageFolder(self,x,z,prefix):
+  
+  def getImageFolder(self,x,z,prefix):
     """Get a unique name for a tile image
     (suitable for use as part of filenames, dictionary keys, etc)"""
     return("%s/%d/%d" % (prefix,z,x))
 
+  def getTileFolderPath(self):
+    """helper function that returns path to the tile folder"""
+    return self.get('tileFolder', 'cache/images')
+
   def imageY(z,extension):
     return (('%d.%s') % (z, extension))
 
@@ -630,6 +659,17 @@ class mapTiles(ranaModule):
     """Wrapper, that makes it possible to use this function from other modules."""
     return getTileUrl(x, y, z, layer)
 
+  def shutdown(self):
+    # shutdown the worker/consumer threads
+    self.shutdownAllThreads = True
+
+    # shutdown the tile loading thread
+    self.tileLoadRequestQueue.put(('shutdown', ()),block=False)
+    # notify the automatic tile download manager thread about the shutdown
+    with self.threadlListCondition:
+      self.threadlListCondition.notifyAll()
+
+
   class tileDownloader(Thread):
     """Downloads an image (in a thread)"""
     def __init__(self,name, x,y,z,layer,layerName,layerType,filename, folder, callback):
