@@ -139,6 +139,17 @@ class options(ranaModule):
 
     # * GPS *
     self.addBoolOption("GPS", "GPSEnabled", "GPS", True, "gpsd:checkGPSEnabled")
+    if self.dmod.locationType() == 'gpsd':
+      knots = "knots per second"
+      meters = "meters per second"
+      if self.device == 'neo':
+        knots = "knots per second (old SHR)"
+        meters = "meters per second (new SHR)"
+      self.addOption("GPSD reports speed in","gpsdSpeedUnit",
+      [('knotsPerSecond', knots),
+       ('metersPerSecond', meters)],
+       "GPS",
+       'knotsPerSecond')
 
     # * screen *
     """only add if supported on device"""
@@ -343,12 +354,12 @@ class options(ranaModule):
             self.set(variable, default)
 
   def save(self):
-    print "saving options"
+    print "options: saving options"
     try:
       f = open(self.optionsFilename(), "w")
       marshal.dump(self.d, f)
       f.close()
-      print "success"
+      print "options: successfully saved"
     except IOError:
       print "Can't save options"
 
