@@ -1,3 +1,4 @@
+from __future__ import with_statement # for python 2.5
 #!/usr/bin/python
 #----------------------------------------------------------------------------
 # Store tiles in a single file.
@@ -19,12 +20,15 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #---------------------------------------------------------------------------
+
 from base_module import ranaModule
 import sqlite3
 import os
 import time
 import glob
 import Queue
+import gtk
+import threading
 from threading import Thread
 
 def getModule(m,d):
@@ -50,6 +54,13 @@ class storeTiles(ranaModule):
     self.lastCommit=time.time()
     self.dirty = set() # a set of connections, that have uncommited data
     self.startLoadingThread()
+    # locks
+    """
+    TODO: this lock might not be needed for python2.6+,
+    as their sqlite version should be thread safe
+    """
+    self.lookupConnectionLock = threading.RLock()
+
 
   def firstTime(self):
     # the config folder should set the tile folder path by now
@@ -119,35 +130,39 @@ class storeTiles(ranaModule):
     stores = self.layers[accessType][dbFolderPath]['stores'] # get a list of cached store connections
 
     lookupCursor = lookupConn.cursor()
-    result = lookupCursor.execute("select store_filename, unix_epoch_timestamp from tiles where z=? and x=? and y=? and extension=?", (z, x, y, extension))
-    if not result.fetchone():
-      # store the tile as its not already in the database
+    with self.lookupConnectionLock:
+      """ just to make sure the access is sequential
+      (due to sqlite in python 2.5 probably not liking concurrent access,
+      resulting in te database becomming unavailable)"""
+      result = lookupCursor.execute("select store_filename, unix_epoch_timestamp from tiles where z=? and x=? and y=? and extension=?", (z, x, y, extension))
+      if not result.fetchone():
+        # store the tile as its not already in the database
 
-      # get a store path
-      size = len(tile)
-      pathToStore = self.getAnAvailableStorePath(folderPrefix, size)
+        # get a store path
+        size = len(tile)
+        pathToStore = self.getAnAvailableStorePath(folderPrefix, size)
 
-      # connect to the store
-      storeConn = self.connectToStore(stores, pathToStore, dbFolderPath, accessType)
+        # connect to the store
+        storeConn = self.connectToStore(stores, pathToStore, dbFolderPath, accessType)
 
-      # store the tile
-      integerTimestamp = int(time.time())
-      try:
-        ## 1. write in the lookup db (its easier to remove pointers to nonexisting stuff than to remove orphaned store items)
-        storeFilename = os.path.basename(pathToStore)
-        lookupQuery="insert into tiles (z, x, y, store_filename, extension, unix_epoch_timestamp) values (?, ?, ?, ?, ?, ?)"
-        lookupCursor = lookupConn.cursor()
-        lookupCursor.execute(lookupQuery,[z,x,y,storeFilename, extension, integerTimestamp])
+        # store the tile
+        integerTimestamp = int(time.time())
+        try:
+          ## 1. write in the lookup db (its easier to remove pointers to nonexisting stuff than to remove orphaned store items)
+          storeFilename = os.path.basename(pathToStore)
+          lookupQuery="insert into tiles (z, x, y, store_filename, extension, unix_epoch_timestamp) values (?, ?, ?, ?, ?, ?)"
+          lookupCursor = lookupConn.cursor()
+          lookupCursor.execute(lookupQuery,[z,x,y,storeFilename, extension, integerTimestamp])
 
-        ## 2. write in the store
-        storeQuery="insert into tiles (z, x, y, tile, extension, unix_epoch_timestamp) values (?, ?, ?, ?, ?, ?)"
-        storeCursor = storeConn.cursor()
-        storeCursor.execute(storeQuery,[z,x,y,sqlite3.Binary(tile), extension, integerTimestamp])
-        self.commitConnections([storeConn,lookupConn])
-      except Exception, e:
-        print "tile already present"
-        print e
-        pass #tile is already present, skip insert
+          ## 2. write in the store
+          storeQuery="insert into tiles (z, x, y, tile, extension, unix_epoch_timestamp) values (?, ?, ?, ?, ?, ?)"
+          storeCursor = storeConn.cursor()
+          storeCursor.execute(storeQuery,[z,x,y,sqlite3.Binary(tile), extension, integerTimestamp])
+          self.commitConnections([storeConn,lookupConn])
+        except Exception, e:
+          print "tile already present"
+          print e
+          pass #tile is already present, skip insert
 
   def commitConnections(self,connections):
     """store connections and commit them once in a while"""
@@ -157,10 +172,14 @@ class storeTiles(ranaModule):
 
   def commitAll(self):
     """commit are uncommited"""
-    while self.dirty:
-      conn = self.dirty.pop()
-      conn.commit()
-    print "committ OK"
+    with self.lookupConnectionLock:
+      """ just to make sure the access is sequential
+      (due to sqlite in python 2.5 probably not liking concurrent access,
+      resulting in te database becomming unavailable)"""
+      while self.dirty:
+        conn = self.dirty.pop()
+        conn.commit()
+      print "storeTiles: sqlite committ OK"
 
 
 
@@ -170,9 +189,13 @@ class storeTiles(ranaModule):
     if pathToStore in stores.keys():
       return stores[pathToStore] # there is already a connection to the store, return it
     else: # create a new connection
-      storeConn = sqlite3.connect(pathToStore) #TODO: add some error handling
-      self.layers[accessType][dbFolderPath]['stores'][pathToStore] = storeConn # cache the connection
-      return storeConn
+      with self.lookupConnectionLock:
+        """ just to make sure the access is sequential
+        (due to sqlite in python 2.5 probably not liking concurrent access,
+        resulting in te database becomming unavailable)"""
+        storeConn = sqlite3.connect(pathToStore) #TODO: add some error handling
+        self.layers[accessType][dbFolderPath]['stores'][pathToStore] = storeConn # cache the connection
+        return storeConn
 
   def getAnAvailableStorePath(self, folderPrefix, size):
     """return a path to a store that can be used to store a tile specified by size"""
@@ -247,37 +270,62 @@ class storeTiles(ranaModule):
 
   def getTile(self, folderPrefix, z, x, y, extension):
     """get a tile"""
-    accessType = "get"
-    dbFolderPath = self.initializeDb(folderPrefix, accessType)
-    if dbFolderPath!=None:
-      lookupConn = self.layers[accessType][dbFolderPath]['lookup'] # connect to the lookup db
-      result = self.getTileFromDb(lookupConn, dbFolderPath, z, x, y, extension)
-      if result: # is the result valid ?
-        resultContent = result.fetchone() # get the result content
-      else:
-        resultContent = None # the result is invalid
-      if resultContent:
-        return resultContent[0] # return the tile image data buffer
-      else:
-        return None # the tile is not stored
+    storageType = self.get('tileStorageType', 'files')
+    if storageType == 'sqlite':
+      accessType = "get"
+      dbFolderPath = self.initializeDb(folderPrefix, accessType)
+      if dbFolderPath!=None:
+        lookupConn = self.layers[accessType][dbFolderPath]['lookup'] # connect to the lookup db
+        result = self.getTileFromDb(lookupConn, dbFolderPath, z, x, y, extension)
+        if result: # is the result valid ?
+          resultContent = result.fetchone() # get the result content
+        else:
+          resultContent = None # the result is invalid
+        if resultContent:
+          # convert the buffer to pixbuf
+          try:
+            pl = gtk.gdk.PixbufLoader()
+            pl.write(resultContent[0])
+            pl.close()
+            pixbuf = pl.get_pixbuf()
+            return pixbuf # return pixbuf containing the tile image
+          except Exception, e:
+            print "loading the image buffer from sqlite to pixbuf failed:%s" % e
+        else:
+          return None # the tile is not stored
+    else: # the only other storage method is currently clasical files storage
+      mapTiles = self.m.get('mapTiles', None)
+      if mapTiles:
+        tileFolderPath = mapTiles.getTileFolderPath()
+        layerFolderAndTileFilename = mapTiles.getImagePath(x,y,z,folderPrefix, extension)
+        tilePath = tileFolderPath + layerFolderAndTileFilename
+        if os.path.exists(tilePath):
+          # load the file to pixbuf and return it
+          return mapTiles.filePath2Pixbuf(tilePath)
+        else:
+          return None # this tile is not locally stored
 
   def getTileFromDb(self, lookupConn, dbFolderPath, z, x, y, extension):
     """get a tile from the database"""
     accessType = "get"
     #look in the lookup db
-    lookupCursor = lookupConn.cursor()
-    lookupResult = lookupCursor.execute("select store_filename, unix_epoch_timestamp from tiles where z=? and x=? and y=? and extension=?", (z, x, y, extension)).fetchone()
-    if lookupResult: # the tile was found in the lookup db
-      # now search in the store
-      storeFilename = lookupResult[0]
-      pathToStore = self.getStorePath(dbFolderPath, storeFilename)
-      stores = self.layers[accessType][dbFolderPath]['stores']
-      connectionToStore = self.connectToStore(stores, pathToStore, dbFolderPath, accessType)
-      storeCursor = connectionToStore.cursor()
-      result = storeCursor.execute("select tile, unix_epoch_timestamp from tiles where z=? and x=? and y=? and extension=?", (z, x, y, extension))
-      return result
-    else: # the tile was not found in the lookup table
-      return None
+    with self.lookupConnectionLock:
+      """ just to make sure the access is sequential
+      (due to sqlite in python 2.5 probably not liking concurrent access,
+      resulting in te database becomming unavailable)"""
+      lookupCursor = lookupConn.cursor()
+      lookupResult = lookupCursor.execute("select store_filename, unix_epoch_timestamp from tiles where z=? and x=? and y=? and extension=?", (z, x, y, extension)).fetchone()
+      if lookupResult: # the tile was found in the lookup db
+        # now search in the store
+        storeFilename = lookupResult[0]
+        pathToStore = self.getStorePath(dbFolderPath, storeFilename)
+        stores = self.layers[accessType][dbFolderPath]['stores']
+        connectionToStore = self.connectToStore(stores, pathToStore, dbFolderPath, accessType)
+        storeCursor = connectionToStore.cursor()
+        result = storeCursor.execute("select tile, unix_epoch_timestamp from tiles where z=? and x=? and y=? and extension=?", (z, x, y, extension))
+        return result
+      else: # the tile was not found in the lookup table
+        return None
 
   def tileExists(self, filePath, folderPrefix, z, x, y, extension, fromThread=False):
     """test if a tile exists
@@ -286,31 +334,48 @@ class storeTiles(ranaModule):
     if storageType == 'sqlite': # we are storing to the database
       dbFolderPath = self.getLayerDbFolderPath(folderPrefix)
       if dbFolderPath!=None: # is the database accessible ?
-        if fromThread: # is this called from a thread ?
-          # due to sqlite quirsk, connections can't be shared between threads
-          lookupDbPath = self.getLookupDbPath(dbFolderPath)
-          lookupConn = sqlite3.connect(lookupDbPath)
-        else:
-          lookupConn = self.layers[dbFolderPath]['lookup'] # connect to the lookup db
-        query = "select store_filename, unix_epoch_timestamp from tiles where z=? and x=? and y=? and extension=?"
-        lookupResult = lookupConn.execute(query,(z, x, y, extension)).fetchone()
-        if fromThread: # tidy up, just to be sure
-          lookupConn.close()
-        if lookupResult:
-          return True # the tile is in the db
-        else:
-            return False # the tile is not in the db
+        with self.lookupConnectionLock:
+          """ just to make sure the access is sequential
+          (due to sqlite in python 2.5 probably not liking concurrent access,
+          resulting in te database becomming unavailable)"""
+          if fromThread: # is this called from a thread ?
+            # due to sqlite quirsk, connections can't be shared between threads
+            lookupDbPath = self.getLookupDbPath(dbFolderPath)
+            lookupConn = sqlite3.connect(lookupDbPath)
+          else:
+            lookupConn = self.layers[dbFolderPath]['lookup'] # connect to the lookup db
+          query = "select store_filename, unix_epoch_timestamp from tiles where z=? and x=? and y=? and extension=?"
+          lookupResult = lookupConn.execute(query,(z, x, y, extension)).fetchone()
+          if fromThread: # tidy up, just to be sure
+            lookupConn.close()
+          if lookupResult:
+            return True # the tile is in the db
+          else:
+              return False # the tile is not in the db
       else:
         return None # we cant decide if a tile is ind the db or not
     else: # we are storing to the filesystem
       return os.path.exists(filePath)
 
+  def startLoadingThread(self):
+    """start the sqlite loading thread"""
+    t = Thread(target=self.worker, name='sqlite tile storage thread')
+    """we need that the worker tidies up,
+       (commits all "dirty" conections)
+       so it should be not daemonic
+       -> but we also cant affort that modrana wont
+       terminate completely
+       """
+    t.setDaemon(True)
+    t.start()
+
+
   def worker(self):
     """this is run by a thread that stores sqlite tiles to a db"""
     while True:
         item = self.sqliteTileQueue.get(block=True)
         if item=='shutdown': # we put this to the queue to announce shutdown
-          print "\nshutdown imminent, commiting all tiles"
+          print "\nshutdown imminent, commiting all uncommited tiles"
           self.commitAll()
           print "\nall tiles commited, breaking, goodbye :)"
           break
@@ -325,7 +390,7 @@ class storeTiles(ranaModule):
           self.storeTile(tile, folderPrefix, z, x, y, extension) # store the tile
           self.sqliteTileQueue.task_done()
         except Exception, e:
-          print "sqlite storage worker -> exception during tile storage:\n%s" % e
+          print "sqlite storage worker : exception during tile storage:\n%s" % e
 
         dt = time.time() - self.lastCommit # check when the last commit was
         if dt>self.commmitInterval:
@@ -333,18 +398,7 @@ class storeTiles(ranaModule):
             self.commitAll() # commit all "dirty" connections
             self.lastCommit = time.time() # update the last commit timestamp
           except Exception, e:
-            print "sqlite storage worker -> exception during mass db commit:\n%s" % e
-
-
-
-  def startLoadingThread(self):
-    """start the sqlite loading thread"""
-    t = Thread(target=self.worker)
-    """we need that the worker tidies up,
-       (commits all "dirty" conections)
-       so it is not daemonic"""
-#    t.daemon = True
-    t.start()
+            print "sqlite storage worker : exception during mass db commit:\n%s" % e
 
 
   def automaticStoreTile(self, tile, folderPrefix, z, x, y, extension, filename, folder, fromThread = False):
